#include <QPushButton>
#include <QSerialPortInfo>
#include <QDebug>
#include <QSerialPortInfo>
#include "SettingsDialogComPort.hpp"
//*********************************************************************************************************************


// окно с выбором COM-порта
SettingsDialogComPort::SettingsDialogComPort(QDialog *parent) : QDialog(parent)
{
  info = new QLabel("info",this);
  portSelectComboBox = new QComboBox(this);

  FillPortsInfo();

//  connect(portSelectComboBox, QOverload<int>::of(&QComboBox::activated),
//          this, &SettingsDialogComPort::FillPortsInfo);

  cancel = new QPushButton("Отмена", this);
  connect(cancel, &QPushButton::clicked, this, &QDialog::close);
  connect(cancel, &QPushButton::clicked, this, &SettingsDialogComPort::ClearSettings);

  apply = new QPushButton("Выбрать", this);
  connect(apply, &QPushButton::clicked, this, &SettingsDialogComPort::ApplySettings);

  buttonsLay = new QHBoxLayout(this);
  buttonsLay->addWidget(apply);
  buttonsLay->addStretch();
  buttonsLay->addWidget(cancel);
  buttonsLay->setContentsMargins(0, 0, 0, 0);
  buttons = new QWidget(this);
  buttons->setLayout(buttonsLay);
  pt = new QLabel("Доступные COM-порты:");

  mainLay = new QVBoxLayout(this);
  mainLay->addWidget(pt);
  mainLay->addWidget(portSelectComboBox);
  mainLay->addWidget(info);
  mainLay->addWidget(buttons);

  setFixedSize(250, 170);      // задать размеры окна диалога выбора последовательного порта

  // убрать кнопку со знаком ? в заголовке окна диалога выбора последовательного порта
  setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);

  setLayout(mainLay);
  setWindowTitle("Подключить COM-порт");
}
//*********************************************************************************************************************


SettingsDialogComPort::~SettingsDialogComPort()
{
    delete info;
    delete portSelectComboBox;
    delete cancel;
    delete apply;
    delete buttonsLay;
    delete buttons;
    delete pt;
    delete mainLay;
}
//*********************************************************************************************************************


// заполнение информации о доступных в системе портах
void SettingsDialogComPort::FillPortsInfo(void)
{
    static const char blankString[] = QT_TRANSLATE_NOOP("SettingsDialog", "N/A");
    const auto infos = QSerialPortInfo::availablePorts();

    QString description[20] = {0};
    QString manufacturer[20] = {0};
    QString serialNumber[20] = {0};

    portSelectComboBox->clear();

    quint8 i = 0;

    for (const QSerialPortInfo &info : infos) {
        i ++;
        QStringList list;
        description[i] = info.description();
        manufacturer[i] = info.manufacturer();
        serialNumber[i] = info.portName();
        list << info.portName()
             << (!description[i].isEmpty() ? description[i] : blankString)
             << (!manufacturer[i].isEmpty() ? manufacturer[i] : blankString)
             << (!serialNumber[i].isEmpty() ? serialNumber[i] : blankString)
             << info.systemLocation()
             << (info.vendorIdentifier() ? QString::number(info.vendorIdentifier(), 16) : blankString)
             << (info.productIdentifier() ? QString::number(info.productIdentifier(), 16) : blankString);

        portSelectComboBox->addItem(list.first(), list);
    }

    portSelectComboBox->setCurrentIndex(currentIndex);

    //    currentPort = portSelectComboBox->currentText();

    //    info->setText(description[currentIndex + 1] + "\n" + manufacturer[currentIndex + 1] + "\n" + currentPort);

    info->setText(description[currentIndex + 1] + "\n" + manufacturer[currentIndex + 1] + "\n" + portSelectComboBox->currentText());

    return;
}
//*********************************************************************************************************************


// выбор порта
void SettingsDialogComPort::ApplySettings()
{
  currentIndex = portSelectComboBox->currentIndex();
  currentPort = portSelectComboBox->currentText();
  close();
  return;
}
//*********************************************************************************************************************


// сброс настроек и закрытие окна выбора COM-порта
void SettingsDialogComPort::ClearSettings()
{
  currentPort = "";
  close();
  return;
}
//*********************************************************************************************************************


// получение имени выбранного порта
QString SettingsDialogComPort::GetPortName()
{
  return currentPort;
}
//*********************************************************************************************************************
