#include "MainWindow.h"
#include "ProtocolCommandSettings.hpp"
//*********************************************************************************************************************


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    QGridLayout *mainLayout = new QGridLayout;
    mainLayout->setSizeConstraint(QLayout::SetMinimumSize);

    menuBar = new QMenuBar;
    menuBar->setStyleSheet("background-color: rgb(225, 225, 225)");
    QColor *color = new QColor("gray");
    QGraphicsDropShadowEffect *shadow1 = new QGraphicsDropShadowEffect;
    shadow1->setColor(*color);
    shadow1->setBlurRadius(5);
    shadow1->setXOffset(5);
    shadow1->setYOffset(5);
    menuBar->setGraphicsEffect(shadow1);
    ConnectAct = new QAction("Подключить COM порт" , this);
    DisConnectAct = new QAction("Отключить COM порт" , this);
    ExitAct = new QAction("Выход из программы", this);

    connectMenu = menuBar->addMenu("Подключение");
    connectMenu->addAction(ConnectAct);
    connectMenu->addAction(DisConnectAct);
    connectMenu->addAction(ExitAct);
    mainLayout->addWidget(menuBar, 0, 0, 1, 2);

    startButton = new QPushButton("Старт проверки");
    stopButton = new QPushButton("Останов проверки");
    clearButton = new QPushButton("Стереть записи лога");
    mainLayout->addWidget(startButton, 0, 3, 1, 5);
    mainLayout->addWidget(stopButton, 0, 9, 1, 5);
    mainLayout->addWidget(clearButton, 0, 15, 1, 5);
    textBrowser = new QTextBrowser;
    mainLayout->addWidget(textBrowser, 1, 0, 1, 20);

    QGraphicsDropShadowEffect *shadow2 = new QGraphicsDropShadowEffect;
    shadow2->setColor(*color);
    shadow2->setBlurRadius(5);
    shadow2->setXOffset(5);
    shadow2->setYOffset(5);
    auto *groupBox1 = new QGroupBox;
    groupBox1->setGraphicsEffect(shadow2);

    progressBar = new QProgressBar;
    progressBar->setVisible(false);
    mainLayout->addWidget(progressBar, 2, 0, 1, 20);

    groupBox1->setLayout(mainLayout);

    setCentralWidget(groupBox1);

    setWindowTitle(tr("Проверка COM портов. " VERSION_TXT VERSION_DIGIT));
#ifdef Q_OS_LINUX
    setFixedSize(GEOMETRY_LINUX);
#endif
#ifdef Q_OS_WIN
    setFixedSize(GEOMETRY_WINDOWS);
#endif
    statusBar();
    QGraphicsDropShadowEffect *shadow3 = new QGraphicsDropShadowEffect;
    shadow3->setColor(*color);
    shadow3->setBlurRadius(3);
    shadow3->setXOffset(3);
    shadow3->setYOffset(3);
    statusBar()->setGraphicsEffect(shadow3);

    stateMachine = new StateMachine();
    // serialPortMain = new ComPort(this);
    serialPortMain = new ComPort();
    serialPortThread = new QThread(this);   // создание потока последовательного порта

    timerReadSerialPort = new QTimer;

    ConnectAct->setEnabled(true);
    DisConnectAct->setEnabled(false);
    startButton->setEnabled(true);
    stopButton->setDisabled(true);

    QMessageBox::warning(Q_NULLPTR,
                         "Внимание!",
                         "Для проверки обмена данными по RS-232"
                         "<br>необходимо установить перемычку"
                         "<br>между контактами разъема COM порта"
                         "<br>DB9(m) 3 TxD и 2 RxD.");

    // общее число байтов даныых в передаваемом блоке должно быть равно или больше 32:
    // 2 байта преамбула (0xAA, 0x55) + 1 байт длина блока данных +
    // 1 байт кода команды + 26 байт данные + 2 байта сигнатура CRC
    // = 32 байта
    static const char data[] = {"\x55\xA9\x56\xA6\x59\xA5\x5A\x9A"
                                "\x65\x99\x66\x95\x69\xAA\x6A\x96"
                                "\x55\xA9\x56\xA6\x59\xA5\x5A\x9A"
                                "\x99\x55"};

    bufferBlock.clear();
    bufferBlock.append(QByteArray::fromRawData(data, (sizeof(data) - 1)));

    // сигнал таймаута таймера проверки чтения ответного сообщения из COM порта
    connect(timerReadSerialPort, &QTimer::timeout, this, &MainWindow::TimerReadSerialPortTimeout);

    // запуск функции OpenPort при выборе пользователем пункта меню "Подключить COM порт"
    connect(ConnectAct, &QAction::triggered, serialPortMain, &ComPort::OpenSettingsDialogComPort);
    // запуск функции ClosePort при выборе пользователем пункта меню "Отключить COM порт"
    connect(DisConnectAct, &QAction::triggered, serialPortMain, &ComPort::ClosePort);
    // выход из программы
    connect(ExitAct, &QAction::triggered, this, &MainWindow::close);

    connect(startButton, &QPushButton::pressed, this, &MainWindow::StartButtonSlot);
    connect(stopButton, &QPushButton::pressed, this, &MainWindow::StopButtonSlot);
    connect(clearButton, &QPushButton::pressed, this, &MainWindow::ClearButtonSlot);

    // запуск функции OpenPort при нажатии экранной кнопки "Старт проверки", если COM порт не был подключен
    connect(this, &MainWindow::OpenSettingsDialogComPortSignal, serialPortMain, &ComPort::OpenSettingsDialogComPort);
    connect(serialPortMain, &ComPort::PortConnectStateSignal, this, &MainWindow::PortConnectStateSlot);

    ConnectFunc();
}
//*********************************************************************************************************************


MainWindow::~MainWindow()
{
    timerReadSerialPort->stop();
    delete timerReadSerialPort;

    if (serialPortThread->isRunning()) {
        serialPortThread->quit();
        serialPortThread->wait();
#if DEBUG
        if (serialPortThread->isFinished())
            qDebug() << SYS_TIME << "Поток последовательного порта завершен!";
#endif
    }
    else {
        // так закрывать и удалять последовательный порт можно
        // только когда закрыт поток, в который перемещен порт
        serialPortMain->ClosePort();
        delete serialPortMain;
    }
    delete serialPortThread;

    delete stateMachine;
    delete startButton;
    delete stopButton;
    delete clearButton;
    delete ConnectAct;
    delete DisConnectAct;
    delete textBrowser;
    delete progressBar;
    delete connectMenu;
    delete menuBar;
}
//*********************************************************************************************************************


void MainWindow::StartButtonSlot(void)
{
    if (!connectComPort) {
        switch (MainWindow::MessageNoConnectComPort()) {
        case 0: // в окне с сообщением нажата экранная кнопка "Да"
            emit OpenSettingsDialogComPortSignal();
            break;
        case 1: //  в окне с сообщением нажата экранная кнопка "Нет"
            break;
        default:
            break;
        }
    }
    else {
        menuBar->setDisabled(true);
        startButton->setDisabled(true);
        stopButton->setEnabled(true);
        progressBar->setVisible(true);
        LoadProcess();
        stateMachine->StateMachineStart();
    }
}
//*********************************************************************************************************************


void MainWindow::StopButtonSlot(void)
{
    stateMachine->StateMachineStop();
    WriteStop();
    numberReadPack = numberWritePack;
}
//*********************************************************************************************************************


void MainWindow::ClearButtonSlot(void)
{
    textBrowser->clear();
}
//*********************************************************************************************************************


// установка значений для машины состояний
void MainWindow::LoadProcess(void)
{
    /***************************************************************************************************
        инициализация таблицы состояний, в каждой строке таблицы три значения:
        1.  имя текущего состояния
        2.  ссылка на функцию, которую небходимо выполнить,
            в некоторых функциях возможно ветвление
            с выполнением нескольких шагов вперед или назад по таблице
        3.  число шагов машины состояний по таблице
            (вперед при положительных значениях, назад при отрицательных или 0 при ожидании событий)
    ***************************************************************************************************/

    StateMachine::transition tableStateInit[] = {
        {START_STATE,                   NULL,                               STEP_FORWARD_1}
        ,{WRITE_BLOCK_BEGIN_STATE,      &StateMachine::StartWork1Signal,    STEP_FORWARD_1} // начало цикла
        ,{READ_BLOCK_WAIT_STATE,        NULL,                               WAIT}
        ,{READ_BLOCK_END_STATE,         &StateMachine::StartWork2Signal,    STEP_BACK_2}    // возврат к началу цикла
        ,{READ_CRC_ERROR_STATE,         &StateMachine::StartWork3Signal,    STEP_FORWARD_1}
        ,{READ_CRC_ERROR_STATE,         &StateMachine::StartWork4Signal,    WAIT}
        ,{READ_DATA_ERROR_STATE,        &StateMachine::StartWork3Signal,    STEP_FORWARD_1}
        ,{READ_DATA_ERROR_STATE,        &StateMachine::StartWork5Signal,    WAIT}
        ,{READ_COMMAND_ERROR_STATE,     &StateMachine::StartWork3Signal,    STEP_FORWARD_1}
        ,{READ_COMMAND_ERROR_STATE,     &StateMachine::StartWork6Signal,    WAIT}
        ,{READ_BLOCK_ERROR_STATE,       &StateMachine::StartWork3Signal,    STEP_FORWARD_1}
        ,{READ_BLOCK_ERROR_STATE,       &StateMachine::StartWork7Signal,    WAIT}
        ,{FATAL_ERROR_STATE,            &StateMachine::StartWork3Signal,    STEP_FORWARD_1}
        ,{FATAL_ERROR_STATE,            &StateMachine::StartWork8Signal,    WAIT}
    };
    stateMachine->LoadTableWork(tableStateInit, sizeof(tableStateInit));
}
//*********************************************************************************************************************


void MainWindow::ConnectFunc(void)
{
    connect(stateMachine, &StateMachine::StartWork1Signal, this, &MainWindow::WriteBlock);
    connect(stateMachine, &StateMachine::StartWork2Signal, this, &MainWindow::ReadBlockEnd);
    connect(stateMachine, &StateMachine::StartWork3Signal, this, &MainWindow::WriteStop);
    connect(stateMachine, &StateMachine::StartWork4Signal, this, &MainWindow::MessageErrorCRC);
    connect(stateMachine, &StateMachine::StartWork5Signal, this, &MainWindow::MessageErrorData);
    connect(stateMachine, &StateMachine::StartWork6Signal, this, &MainWindow::MessageErrorCommand);
    connect(stateMachine, &StateMachine::StartWork7Signal, this, &MainWindow::ReadBlockError);
    connect(stateMachine, &StateMachine::StartWork8Signal, this, &MainWindow::FatalError);
}
//*********************************************************************************************************************


// запись 26 байт данных + 1 байт кода команды
void MainWindow::WriteBlock(void)
{
    QByteArray Command;

    Command.clear();

    progressValue ++;
    if (progressValue == 100)
        progressValue = 0;

    quint8 n = bufferBlock.size(), d = 0;
    static quint8 i = 0;
    if (i == n)
        i = 0;
    d = bufferBlock.at(i);
    d ++;
    bufferBlock.remove(i, 1);
    bufferBlock.insert(i, d);
    i ++;

    numberWritePack ++;
    textBrowser->setTextColor("green");
    textBrowser->append(SYS_TIME + "   Номер переданного блока  " + QString::number(numberWritePack));
    textBrowser->setTextColor("brown");
    textBrowser->append(CharToHex(bufferBlock));

    progressBar->setValue(progressValue);
    statusBar()->showMessage(sysTime + "      COM порт " +
                             stringComPortName +
                             " подключен."
                             "Скорость обмена данными " + QString::number(BAUD_RATE) + " бит/сек.", 1000000);

    Command.resize(1);
    Command[0] = static_cast<char>(WRITE_BLOCK_REQUEST);
    Command.insert(1, bufferBlock);

    emit SendDataToSerialPortSignal(Command);
    TimerReadSerialPortStart(COMPORT_TIMEOUT);
}
//*********************************************************************************************************************


void MainWindow::ReadBlockEnd(void)
{
    numberReadPack ++;
    textBrowser->setTextColor("green");
    if (numberReadPack != 1)
        if (!(numberReadPack % 5000))
            textBrowser->clear();
    textBrowser->append(SYS_TIME + "   Номер принятого блока      " + QString::number(numberReadPack));
    textBrowser->setTextColor("blue");
    textBrowser->append(CharToHex(readData));
}
//*********************************************************************************************************************


void MainWindow::WriteStop(void)
{
    timerReadSerialPort->stop();
    menuBar->setEnabled(true);
    startButton->setEnabled(true);
    stopButton->setDisabled(true);
}
//*********************************************************************************************************************


void MainWindow::FatalError(void)
{
    stateMachine->StateMachineStop();
    textBrowser->setTextColor("red");
    textBrowser->append("Фатальная ошибка! ");
    QMessageBox::critical(nullptr, "Ошибка!", "Фатальная ошибка! ");
}
//*********************************************************************************************************************


void MainWindow::ReadBlockError(void)
{
    stateMachine->StateMachineStop();
    // serialPortMain->serialPort->clear(QSerialPort::AllDirections);
    textBrowser->setTextColor("red");
    textBrowser->append("Ошибка! Не приняты данные из COM порта! ");
    QMessageBox::critical(nullptr, "Ошибка!", "Не приняты данные из COM порта! ");
}
//*********************************************************************************************************************


void MainWindow::MessageErrorCRC(void)
{
    stateMachine->StateMachineStop();
    QMessageBox::critical(nullptr, "Ошибка!", "Неверна контрольная сумма<br>принятых из COM порта данных! ");
}
//*********************************************************************************************************************


void MainWindow::MessageErrorData(void)
{
    stateMachine->StateMachineStop();
    QMessageBox::critical(nullptr, "Ошибка!", "Принятые из COM порта данные<br>не соответствуют переданным! ");
}
//*********************************************************************************************************************


void MainWindow::MessageErrorCommand(void)
{
    stateMachine->StateMachineStop();
    QMessageBox::critical(nullptr, "Ошибка!", "Код команды принят с искажениями! ");
}
//*********************************************************************************************************************


// запуск таймера проверки чтения ответного блока данных из COM порта
void MainWindow::TimerReadSerialPortStart(quint16 timeOut)
{
    if (!timerReadSerialPort->isActive()) {
        timerReadSerialPort->start(timeOut);
        timerReadSerialPort->setSingleShot(true);
    }
}
//*********************************************************************************************************************


// обработка таймаута приема ответного блока данных из COM порта
void MainWindow::TimerReadSerialPortTimeout(void)
{
    switch (stateMachine->CurrentStateGet()) {
        case READ_BLOCK_WAIT_STATE:
            stateMachine->EventSet(EVENT_READ_BLOCK_ERROR);
            break;
        default:
            stateMachine->EventSet(EVENT_FATAL_ERROR);
            break;
    }
}
//*********************************************************************************************************************


int MainWindow::MessageNoConnectComPort(void)
{
    QMessageBox msg = QMessageBox(QMessageBox::Question,
                                  "Сделайте выбор",
                                  "COM порт не подключен в меню \"Подключение\".<br>"
                                  "Если хотите подключить - нажмите \"Да\".<br>"
                                  "Если нет, нажмите \"Нет\".",
                                  QMessageBox::NoButton,
                                  nullptr);

    msg.addButton(tr("Да"), QMessageBox::YesRole);
    msg.addButton(tr("Нет"), QMessageBox::NoRole);

    return msg.exec();
}
//*********************************************************************************************************************


// установка доступности органов управления программы в зависимости
// от состояния последовательного порта - подключен/не подключен
// перемещение экземпляров классов ComPort и QSerialPort в отдельный поток
void MainWindow::PortConnectStateSlot(bool signPortConn)
{
    stringComPortName = serialPortMain->serialPort->portName();
    if (signPortConn) {
        ConnectAct->setEnabled(false);
        DisConnectAct->setEnabled(true);
        sysTime = SYS_TIME;
        statusBar()->showMessage(sysTime + "      COM порт " +
                                 stringComPortName +
                                 " подключен."
                                 "Скорость обмена данными " + QString::number(BAUD_RATE) + " бит/сек.", 0);
        connectComPort = true;

        serialPortMain->moveToThread(serialPortThread);                                 // перемещение экземпляра класса ComPort в поток
        serialPortMain->serialPort->moveToThread(serialPortThread);                     // перемещение экземпляра класса QSerialPort в поток
        //        serialPortThread->setPriority(QThread::TimeCriticalPriority);         // установка приоритета (можно при запуске потока)
        connect(serialPortThread, &QThread::started, this, &MainWindow::ConnnectProcessPortSlot);   // переназначение метода run
        connect(serialPortMain, &ComPort::finishedPort, serialPortThread, &QThread::quit);          // переназначение метода выход
        connect(serialPortThread, &QThread::finished, serialPortMain, &QObject::deleteLater);       // удалить порт
        // ВАЖНО: запускать поток последовательного порта после всех connect(serialPortThread...)!!!
        serialPortThread->start(QThread::TimeCriticalPriority);         // установка приоритета при запуске потока
    }
    else {
        ConnectAct->setEnabled(true);
        DisConnectAct->setEnabled(false);
        statusBar()->showMessage(SYS_TIME + "      COM порт " + stringComPortNameSawe + " отключен.", 0);
        connectComPort = false;
        progressValue = 0;
        progressBar->reset();
        progressBar->setVisible(false);
        disconnect(serialPortMain, &ComPort::finishedPort, serialPortThread, &QThread::quit);          // переназначение метода выход
        disconnect(serialPortThread, &QThread::finished, serialPortMain, &QObject::deleteLater);       // удалить порт
    }
    stringComPortNameSawe = stringComPortName;
}
//*********************************************************************************************************************


// создание соединения для передачи данных для записи в последовательный порт
void MainWindow::ConnnectProcessPortSlot(void)
{
#if DEBUG
    if (serialPortThread->isRunning())
        qDebug() << SYS_TIME << "Поток последовательного порта запущен!";
#endif
    // передача данных для записи в последовательный порт
    connect(this, &MainWindow::SendDataToSerialPortSignal, serialPortMain, &ComPort::WriteData);
    // прочитанные из приемного буфера последовательного порта
    // данные передаются в функцию расшифровки принятых сообщений
    connect(serialPortMain, &ComPort::RecivePacketSignal, this, &MainWindow::DecodeReadData);
}
//*********************************************************************************************************************


// расшифровка принятых по интерфейсу RS-232 сообщений
void MainWindow::DecodeReadData(QByteArray data)
{
    quint8 dataLength = data.at(2);
    quint8 codeCommand = data.at(3);

#if DEBUG_ARRAY_RX_2
    qDebug() << "Функция DecodeReadData:";
    qDebug() << "Размер данных из принятого массива =" << dataLength;
    qDebug() << "Вычисленный размер принятого массива =" << data.size();
    qDebug() << "Принятый массив data =";
    qDebug() << CharToHex(data);
#endif

    quint16 crcRecive = 0x0000;
    crcRecive = (quint8) (data.at(dataLength - 2));
    crcRecive <<= 8;
    crcRecive |= (quint8) data.at(dataLength - 1);

    data.resize(dataLength - 2);
    quint16 crcCalc = serialPortMain->CalcCRC16(&data, 0);

#if DEBUG_ARRAY_RX_2
    qDebug() << "crc принятое =" << crcRecive;
    qDebug() << "crc вычисленное =" << crcCalc;
#endif

    if (crcRecive != crcCalc) {
        stateMachine->EventSet(EVENT_READ_CRC_ERROR);
        return;
    }

    data.resize(dataLength + 2);

    switch (codeCommand) {
        case WRITE_BLOCK_REQUEST:
            if (stateMachine->CurrentStateGet() == READ_BLOCK_WAIT_STATE) {
                timerReadSerialPort->stop();
                data.remove(0, 4);
                data.chop(2);
                data = data.left(bufferBlock.size());
                if (data == bufferBlock) {
                    readData = data;
                    stateMachine->EventSet(EVENT_READ_MINI_BLOCK);
                }
                else
                    stateMachine->EventSet(EVENT_READ_DATA_ERROR);
            }
            break;
        default:
            stateMachine->EventSet(EVENT_READ_COMMAND_ERROR);
            break;
    }
    return;
}
//*********************************************************************************************************************
