#ifndef MAINWINDOW_H
#define MAINWINDOW_H
//*********************************************************************************************************************

#include <QMainWindow>
#include <QWidget>
#include <QThread>
#include <QTime>
#include <QTimer>
#include <QGroupBox>
#include <QGraphicsDropShadowEffect>
#include <QDebug>
#include <QMenuBar>
#include <QStatusBar>
#include <QMessageBox>
#include <QTextBrowser>
#include <QProgressBar>
#include <QPushButton>
#include "ComPort.hpp"
#include "StateMachine.h"
//*********************************************************************************************************************


#define VERSION_TXT         "Версия "
#define VERSION_DIGIT       "00.04"

#define DEBUG               false //true //

// NOTE размеры главного окна программы в ОС WINDOWS
#define GEOMETRY_WINDOWS    920, 595
// NOTE размеры главного окна программы в ОС LINUX
#define GEOMETRY_LINUX      1055, 630

#define SYS_TIME            QTime::currentTime().toString("HH:mm:ss:zzz")

// состояния программы
typedef enum {
    START_STATE = 0,                    // состояние при старте программы
    WRITE_BLOCK_BEGIN_STATE,            // запись блока данных в COM порт
    READ_BLOCK_WAIT_STATE,              // ожидание приема блока данных из COM порта
    READ_BLOCK_END_STATE,               // закончено чтение блока данных из COM порта
    WRITE_STOP_STATE,                   // останов записи в COM порт
    READ_CRC_ERROR_STATE,               // несовпадение переданной и принятой сигнатуры CRC
    READ_DATA_ERROR_STATE,              // несовпадение переданных и принятых данных
    READ_COMMAND_ERROR_STATE,           // принят неизвестный код команды
    READ_BLOCK_ERROR_STATE,             // не был принят блок данных в течение заданного времени
    FATAL_ERROR_STATE                   // таймаут приема ответнного блока данных из COM порта
                                        // закончился при состоянии, отличном от состояния
                                        // ожидания приема блока данных из COM порта
} enumState;

#define EVENT_READ_MINI_BLOCK           1   // принят мини блок
#define EVENT_READ_CRC_ERROR            2
#define EVENT_READ_DATA_ERROR           4
#define EVENT_READ_COMMAND_ERROR        6
#define EVENT_READ_BLOCK_ERROR          8
#define EVENT_START_STATE               -2
#define EVENT_FATAL_ERROR               10


class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

signals:
    void OpenSettingsDialogComPortSignal(void);
    void SendDataToSerialPortSignal(QByteArray data);
    void WriteBlockSignal(void);

public slots:

private slots:
    void TimerReadSerialPortTimeout(void);
    void StartButtonSlot(void);
    void StopButtonSlot(void);
    void ClearButtonSlot(void);
    void ConnnectProcessPortSlot(void);
    void PortConnectStateSlot(bool);

private:
    QMenu       *connectMenu;
    QMenuBar    *menuBar;
    QAction     *ConnectAct;
    QAction     *DisConnectAct;
    QAction     *ExitAct;
    QPushButton *startButton;
    QPushButton *stopButton;
    QPushButton *clearButton;
    QTextBrowser *textBrowser;
    QProgressBar *progressBar;

    StateMachine *stateMachine;
    ComPort *serialPortMain;
    QThread *serialPortThread;
    QTimer *timerReadSerialPort;
    qint64 numberWritePack = 0;
    quint64 numberReadPack = 0;
    QByteArray bufferBlock;
    quint8 connectComPort = false;
    QByteArray readData;
    qint8 progressValue = 0;

    QString sysTime;

    QString stringComPortName;
    QString stringComPortNameSawe;

    void TimerReadSerialPortStart(quint16);
    int MessageNoConnectComPort(void);
    void WriteBlock(void);
    // расшифровка принятых сообщений
    void DecodeReadData(QByteArray data);
    void ReadBlockEnd(void);
    void WriteStop(void);
    void ReadBlockError();
    void FatalError();

    void LoadProcess(void);
    void ConnectFunc(void);

    void MessageErrorCRC(void);
    void MessageErrorData(void);
    void MessageErrorCommand(void);
};
//*********************************************************************************************************************


static inline QByteArray CharToHex(QByteArray data)
{
    QByteArray array;

    for (quint16 i = 0, j = 0; i < data.size(); i ++, j += 5) {
        quint8 n = data.at(i);
        if (n & 0xF0)
            array.insert(j, QString("0x").toUtf8());
        else
            array.insert(j, QString("0x0").toUtf8());
        array.append(QString("%1").arg(n, 0, 16).toUpper().toUtf8());
        if (i != (data.size() - 1))
            array.append(QString(":").toUtf8());
    }
    return array;
}
//*********************************************************************************************************************


#endif // MAINWINDOW_H
//*********************************************************************************************************************
