#include <QDebug>
#include <QThread>
#include "StateMachine.h"
//*********************************************************************************************************************


StateMachine::StateMachine(QObject *parent) : QObject(parent)
{
    timerWork = new QTimer;         // создание таймера обработки состояний
    timerWorkThread = new QThread;  // создание потока таймера обработки состояний
    currentState = WAIT;      // состояние при старте программы
    timerWork->start(STATE_MACHINE_CYCLE);
    timerWork->moveToThread(timerWorkThread);      // перемещение класса таймера в поток
    connect(timerWorkThread, &QThread::started, this, &StateMachine::ConnnectTimerWorkSlot);    // переназначение метода run
    connect(timerWorkThread, &QThread::finished, timerWork, &QObject::deleteLater);             // удалить таймер при закрытии потока
    timerWorkThread->start(QThread::TimeCriticalPriority);    // установка приоритета при запуске потока
}
//*********************************************************************************************************************


StateMachine::~StateMachine()
{
    if (timerWorkThread->isRunning()) {
        timerWorkThread->quit();
        timerWorkThread->wait();
        timerWork->stop();
    }
}
//*********************************************************************************************************************


// обработка состояний
void StateMachine::StateMachineWork(void)
{
    if (startState) {
        if (tableStateWork[tablePointer].worker)                    // если существует указатель на функцию
            (this->*tableStateWork[tablePointer].worker) ();        // выполнение функции

        if (eventSM) {                                              // если в функции произошло событие
            tablePointer += eventSM;                                // изменяется текущий номер строки таблицы в
            eventSM = 0;                                            // соответствии с числом, заданным в событии
        }
        else    // если указатель на строку таблицы машины состояний не был задан событием
            tablePointer += tableStateWork[tablePointer].nextStep;  // к номеру строки таблицы добавляется
                                                                    // число шагов из текущей строки таблицы
                                                                    // число шагов может быть отрицательным
                                                                    // для организации цикла
        currentState = tableStateWork[tablePointer].newState;       // текущее состояние
    }

#if DEBUG_STATE_MACHINE
    qDebug() << "===== Выполнен шаг по таблице машины состояний =====";
    qDebug() << "nextStep            =" << tableStateWork[tablePointer].nextStep;
    qDebug() << "tablePointer        =" << tablePointer;
    qDebug() << "currentState        =" << currentState;
#endif
}
//*********************************************************************************************************************


void StateMachine::ConnnectTimerWorkSlot(void)
{
#if DEBUG
    if (timerWorkThread->isRunning())
        qDebug() << SYS_TIME << "Поток рабочего таймера запущен!";
#endif
    // вызов обработчика машины состояний по таймауту таймера
    connect(timerWork, &QTimer::timeout, this, &StateMachine::StateMachineWork);
}
//*********************************************************************************************************************


// загрузка рабочей таблицы состояний
void StateMachine::LoadTableWork(transition *tabSt, quint64 size)
{
    for (quint8 i = 0; i < TAB_STAT_PROC_LEN; ++ i)
        tableStateWork[i] = {WAIT, NULL, WAIT};
    for (quint8 i = 0; i < size / sizeof(transition); ++ i)
        tableStateWork[i] = tabSt[i];
    eventSM = WAIT;
    tablePointer = WAIT;
    currentState = WAIT;
}
//*********************************************************************************************************************


// старт обработчика машины состояний
void StateMachine::StateMachineStart(void)
{
    startState = true;
}
//*********************************************************************************************************************


// останов обработчика машины состояний
void StateMachine::StateMachineStop(void)
{
    startState = false;
}
//*********************************************************************************************************************


// обнуление указателя на строку таблицы
void StateMachine::TablePointerReset(void)
{
    tablePointer = WAIT;
}
//*********************************************************************************************************************


// установка значения события
void StateMachine::EventSet(quint8 value)
{
    eventSM = value;
}
//*********************************************************************************************************************


// установка значения текущего состояния
void StateMachine::CurrentStateSet(quint8 value)
{
    currentState = value;
}
//*********************************************************************************************************************


// возврат значения текущего состояния
quint8 StateMachine::CurrentStateGet(void)
{
    return currentState;
}
//*********************************************************************************************************************
