﻿#ifndef STATEMACHINE_H
#define STATEMACHINE_H
//*********************************************************************************************************************


#include <QObject>
#include <QTimer>
//*********************************************************************************************************************


#define DEBUG_STATE_MACHINE     false //true //

#define STATE_MACHINE_CYCLE     1   // мс, период вызова обработчика машины состояний

#define TAB_STAT_PROC_LEN       50

#define WAIT                0       // ожидание события или бездействие программы
#define STEP_FORWARD_1      1       // переход к следующему состоянию
#define STEP_FORWARD_2      2       // 2 шага вперед по таблице состояний
#define STEP_FORWARD_3      3
#define STEP_FORWARD_4      4
#define STEP_FORWARD_5      5
#define STEP_FORWARD_6      6
#define STEP_FORWARD_7      7
#define STEP_FORWARD_8      8
#define STEP_FORWARD_9      9
#define STEP_FORWARD_10     10
#define STEP_FORWARD_11     11
#define STEP_FORWARD_12     12
#define STEP_FORWARD_13     13
#define STEP_FORWARD_14     14
#define STEP_FORWARD_15     15
#define STEP_BACK_1         -1      // 1 шаг назад по таблице состояний
#define STEP_BACK_2         -2      // 2 шага назад по таблице состояний
#define STEP_BACK_3         -3
#define STEP_BACK_4         -4
#define STEP_BACK_5         -5
#define STEP_BACK_6         -6
#define STEP_BACK_7         -7
#define STEP_BACK_8         -8
#define STEP_BACK_9         -9
#define STEP_BACK_10        -10     // 10 шагов назад по таблице состояний
#define STEP_BACK_11        -11
#define STEP_BACK_12        -12
#define STEP_BACK_13        -13
#define STEP_BACK_14        -14
#define STEP_BACK_15        -15


class StateMachine : public QObject
{
    Q_OBJECT
public:
    explicit StateMachine(QObject *parent = nullptr);
    ~StateMachine();

    typedef void (StateMachine::*callback) (void);

    typedef struct
    {
        quint16     newState;
        callback    worker;
        qint8       nextStep;
    } transition;

    void LoadTableWork(transition *, quint64);
    void StateMachineStart(void);
    void StateMachineStop(void);
    void TablePointerReset(void);
    void EventSet(quint8);
    void CurrentStateSet(quint8);
    quint8 CurrentStateGet(void);

signals:
    void StartWork1Signal(void);
    void StartWork2Signal(void);
    void StartWork3Signal(void);
    void StartWork4Signal(void);
    void StartWork5Signal(void);
    void StartWork6Signal(void);
    void StartWork7Signal(void);
    void StartWork8Signal(void);
    void StartWork9Signal(void);
    void StartWork10Signal(void);
    void StartWork11Signal(void);
    void StartWork12Signal(void);
    void StartWork13Signal(void);
    void StartWork14Signal(void);
    void StartWork15Signal(void);

private slots:
    void ConnnectTimerWorkSlot(void);

private:
    QTimer *timerWork;                  // таймер периода вызова обработчика машины состояний
    QThread *timerWorkThread;           // поток таймера периода вызова обработчика машины состояний

    transition tableStateWork[TAB_STAT_PROC_LEN] = {};

    quint8 tablePointer = 0;            // текущий номер строки таблицы состояний
    quint8 eventSM = 0;                 // событие машины состояний
    quint8 currentState;                // состояние программы
    bool startState = false;            // режим работы машины состояний

    void StateMachineWork(void);
};
//*********************************************************************************************************************


#endif // STATEMACHINE_H
//*********************************************************************************************************************
